@@ -19,6 +19,7 @@ import os
 import warnings
 import tempfile
 import shutil
+import copy
 from functools import update_wrapper, wraps
 from pathlib import Path
 from typing import Callable, Iterable, TypeVar, Any, cast
@@ -41,6 +42,18 @@ from .outschema_todict import create_outschema_dict, merge_schema_dicts
 
 PACKAGE_DIRECTORY = Path(__file__).parent.resolve()
 
+EMPTY_TAG_INFO: TagInfo = {
+    'name': None,  #type: ignore[typeddict-item]
+    'attribs': CaseInsensitiveFrozenSet(),
+    'optional_attribs': CaseInsensitiveDict(),
+    'optional': CaseInsensitiveFrozenSet(),
+    'order': [],
+    'several': CaseInsensitiveFrozenSet(),
+    'simple': CaseInsensitiveFrozenSet(),
+    'complex': CaseInsensitiveFrozenSet(),
+    'text': CaseInsensitiveFrozenSet()
+}
+
 
 class NoPathFound(ValueError):
     """
@@ -127,12 +140,13 @@ def schema_dict_version_dispatch(output_schema: bool = False) -> Callable[[F], S
                 if min_version is None and max_version is None:
                     raise ValueError('Either a minimum or maximum version has to be given')
 
-                if min_version is not None and max_version is not None:
-                    cond_func = lambda version: min_version_tuple <= version <= max_version_tuple
-                elif min_version is not None:
-                    cond_func = lambda version: version >= min_version_tuple
-                else:
-                    cond_func = lambda version: version <= max_version_tuple
+                def cond_func(version):
+                    if min_version is not None and max_version is not None:
+                        return min_version_tuple <= version <= max_version_tuple
+                    elif min_version is not None:
+                        return version >= min_version_tuple
+                    else:
+                        return version <= max_version_tuple
 
                 registry[cond_func] = func
 
@@ -529,44 +543,27 @@ class SchemaDict(LockableDict):
                                       ' since no tag or info entries are defined')
 
         paths = self._find_paths(name, self._tag_entries, contains=contains, not_contains=not_contains)
-        if len(paths) == 0:
-            raise NoPathFound(f'The tag {name} has no possible paths with the current specification.\n'
-                              f'contains: {contains}, not_contains: {not_contains}')
-
-        EMPTY_TAG_INFO: TagInfo = {
-            'attribs': CaseInsensitiveFrozenSet(),
-            'optional_attribs': CaseInsensitiveDict(),
-            'optional': CaseInsensitiveFrozenSet(),
-            'order': [],
-            'several': CaseInsensitiveFrozenSet(),
-            'simple': CaseInsensitiveFrozenSet(),
-            'complex': CaseInsensitiveFrozenSet(),
-            'text': CaseInsensitiveFrozenSet()
-        }
+        if parent:
+            paths = [split_off_tag(path)[0] for path in paths]
 
         tag_info = None
         for path in paths:
 
-            if parent:
-                path, _ = split_off_tag(path)
-
-            entry = None
+            entry = copy.deepcopy(EMPTY_TAG_INFO)
+            entry['name'] = split_off_tag(path)[1]
             for info_entry in self._info_entries:
                 if path in self[info_entry]:
                     entry = self[info_entry][path]
-            if entry is None:
-                entry = EMPTY_TAG_INFO
 
-            if tag_info is not None:
-                if entry != tag_info:
-                    raise NoUniquePathFound(f'Differing tag_info for the found with the current specification\n'
-                                            f'contains: {contains}, not_contains: {not_contains}\n'
-                                            f'These are possible:  {paths}')
-            else:
-                tag_info = entry
+            if tag_info is not None and entry != tag_info:
+                raise NoUniquePathFound(f'Differing tag_info for the found with the current specification\n'
+                                        f'contains: {contains}, not_contains: {not_contains}\n'
+                                        f'These are possible:  {paths}')
+            tag_info = entry
 
         if tag_info is None:
-            raise ValueError(f'No tag info found for paths: {paths}')
+            raise NoPathFound(f'The tag {name} has no possible paths with the current specification.\n'
+                              f'contains: {contains}, not_contains: {not_contains}, parent: {parent}')
 
         return tag_info
 
